@@ -1,3 +1,4 @@
+from __future__ import division
 from utm.error import OutOfRangeError
 
 # For most use cases in this module, numpy is indistinguishable
@@ -16,7 +17,7 @@ K0 = 0.9996
 E = 0.00669438
 E2 = E * E
 E3 = E2 * E
-E_P2 = E / (1.0 - E)
+E_P2 = E / (1 - E)
 
 SQRT_E = mathlib.sqrt(1 - E)
 _E = (1 - SQRT_E) / (1 + SQRT_E)
@@ -30,10 +31,10 @@ M2 = (3 * E / 8 + 3 * E2 / 32 + 45 * E3 / 1024)
 M3 = (15 * E2 / 256 + 45 * E3 / 1024)
 M4 = (35 * E3 / 3072)
 
-P2 = (3. / 2 * _E - 27. / 32 * _E3 + 269. / 512 * _E5)
-P3 = (21. / 16 * _E2 - 55. / 32 * _E4)
-P4 = (151. / 96 * _E3 - 417. / 128 * _E5)
-P5 = (1097. / 512 * _E4)
+P2 = (3 / 2 * _E - 27 / 32 * _E3 + 269 / 512 * _E5)
+P3 = (21 / 16 * _E2 - 55 / 32 * _E4)
+P4 = (151 / 96 * _E3 - 417 / 128 * _E5)
+P5 = (1097 / 512 * _E4)
 
 R = 6378137
 
@@ -50,27 +51,27 @@ def in_bounds(x, lower, upper, upper_strict=False):
     return lower <= x <= upper
 
 
-def check_valid_zone(zone_number, zone_letter):
+def check_valid_zone_letter(zone_letter):
+    zone_letter = zone_letter.upper()
+    if not 'C' <= zone_letter <= 'X' or zone_letter in ['I', 'O']:
+        raise OutOfRangeError('zone letter out of range (must be between C and X)')
+
+
+def check_valid_zone_number(zone_number):
     if not 1 <= zone_number <= 60:
         raise OutOfRangeError('zone number out of range (must be between 1 and 60)')
 
-    if zone_letter:
-        zone_letter = zone_letter.upper()
 
-        if not 'C' <= zone_letter <= 'X' or zone_letter in ['I', 'O']:
-            raise OutOfRangeError('zone letter out of range (must be between C and X)')
+def check_valid_zone(zone_number, zone_letter):
+    check_valid_zone_number(zone_number)
+    if zone_letter:
+        check_valid_zone_letter(zone_letter)
 
 
 def mixed_signs(x):
     return use_numpy and mathlib.min(x) < 0 and mathlib.max(x) >= 0
 
 
-def negative(x):
-    if use_numpy:
-        return mathlib.max(x) < 0
-    return x < 0
-
-
 def mod_angle(value):
     """Returns angle in radians to be between -pi and pi"""
     return (value + mathlib.pi) % (2 * mathlib.pi) - mathlib.pi
@@ -96,7 +97,8 @@ def to_latlon(easting, northing, zone_number, zone_letter=None, northern=None, s
             designators can be seen in [1]_
 
         northern: bool
-            You can set True or False to set this parameter. Default is None
+            You can set True (North) or False (South) as an alternative to
+            providing a zone letter. Default is None
 
         strict: bool
             Raise an OutOfRangeError if outside of bounds
@@ -115,7 +117,6 @@ def to_latlon(easting, northing, zone_number, zone_letter=None, northern=None, s
     """
     if not zone_letter and northern is None:
         raise ValueError('either zone_letter or northern needs to be set')
-
     elif zone_letter and northern is not None:
         raise ValueError('set either zone_letter or northern, but not both')
 
@@ -124,18 +125,15 @@ def to_latlon(easting, northing, zone_number, zone_letter=None, northern=None, s
             raise OutOfRangeError('easting out of range (must be between 100,000 m and 999,999 m)')
         if not in_bounds(northing, 0, 10000000):
             raise OutOfRangeError('northing out of range (must be between 0 m and 10,000,000 m)')
-    
+
     check_valid_zone(zone_number, zone_letter)
-    
+
     if zone_letter:
         zone_letter = zone_letter.upper()
         northern = (zone_letter >= 'N')
 
     x = easting - 500000
-    y = northing
-
-    if not northern:
-        y -= 10000000
+    y = northing if northern else northing - 10000000
 
     m = y / K0
     mu = m / (R * M1)
@@ -171,9 +169,9 @@ def to_latlon(easting, northing, zone_number, zone_letter=None, northern=None, s
     d5 = d4 * d
     d6 = d5 * d
 
-    latitude = (p_rad - (p_tan / r) *
-                (d2 / 2 -
-                 d4 / 24 * (5 + 3 * p_tan2 + 10 * c - 4 * c2 - 9 * E_P2)) +
+    latitude = p_rad - (p_tan / r) * (
+                 d2 / 2 -
+                 d4 / 24 * (5 + 3 * p_tan2 + 10 * c - 4 * c2 - 9 * E_P2) +
                  d6 / 720 * (61 + 90 * p_tan2 + 298 * c + 45 * p_tan4 - 252 * E_P2 - 3 * c2))
 
     longitude = (d -
@@ -186,7 +184,7 @@ def to_latlon(easting, northing, zone_number, zone_letter=None, northern=None, s
             mathlib.degrees(longitude))
 
 
-def from_latlon(latitude, longitude, force_zone_number=None, force_zone_letter=None):
+def from_latlon(latitude, longitude, force_zone_number=None, force_zone_letter=None, force_northern=None):
     """This function converts Latitude and Longitude to UTM coordinate
 
         Parameters
@@ -206,6 +204,11 @@ def from_latlon(latitude, longitude, force_zone_number=None, force_zone_letter=N
             You may force conversion to be included within one UTM zone
             letter.  For more information see utmzones [1]_
 
+        force_northern: bool
+            You can set True (North) or False (South) as an alternative to
+            forcing with a zone letter. When set, the returned zone_letter will
+            be None. Default is None
+
         Returns
         -------
         easting: float or NumPy array
@@ -225,10 +228,12 @@ def from_latlon(latitude, longitude, force_zone_number=None, force_zone_letter=N
 
        .. _[1]: http://www.jaworski.ca/utmzones.htm
     """
-    if not in_bounds(latitude, -80.0, 84.0):
+    if not in_bounds(latitude, -80, 84):
         raise OutOfRangeError('latitude out of range (must be between 80 deg S and 84 deg N)')
-    if not in_bounds(longitude, -180.0, 180.0):
+    if not in_bounds(longitude, -180, 180):
         raise OutOfRangeError('longitude out of range (must be between 180 deg W and 180 deg E)')
+    if force_zone_letter and force_northern is not None:
+        raise ValueError('set either force_zone_letter or force_northern, but not both')
     if force_zone_number is not None:
         check_valid_zone(force_zone_number, force_zone_letter)
 
@@ -245,11 +250,16 @@ def from_latlon(latitude, longitude, force_zone_number=None, force_zone_letter=N
     else:
         zone_number = force_zone_number
 
-    if force_zone_letter is None:
+    if force_zone_letter is None and force_northern is None:
         zone_letter = latitude_to_zone_letter(latitude)
     else:
         zone_letter = force_zone_letter
 
+    if force_northern is None:
+        northern = (zone_letter >= 'N')
+    else:
+        northern = force_northern
+
     lon_rad = mathlib.radians(longitude)
     central_lon = zone_number_to_central_longitude(zone_number)
     central_lon_rad = mathlib.radians(central_lon)
@@ -276,10 +286,10 @@ def from_latlon(latitude, longitude, force_zone_number=None, force_zone_letter=N
     northing = K0 * (m + n * lat_tan * (a2 / 2 +
                                         a4 / 24 * (5 - lat_tan2 + 9 * c + 4 * c**2) +
                                         a6 / 720 * (61 - 58 * lat_tan2 + lat_tan4 + 600 * c - 330 * E_P2)))
-
-    if mixed_signs(latitude):
+    check_signs = force_northern is None and force_zone_letter is None
+    if check_signs and mixed_signs(latitude):
         raise ValueError("latitudes must all have the same sign")
-    elif negative(latitude):
+    elif not northern:
         northing += 10000000
 
     return easting, northing, zone_number, zone_letter
@@ -306,9 +316,13 @@ def latlon_to_zone_number(latitude, longitude):
         if isinstance(longitude, mathlib.ndarray):
             longitude = longitude.flat[0]
 
+    # Normalize longitude to be in the range [-180, 180)
+    longitude = (longitude % 360 + 540) % 360 - 180
+
+    # Special zone for Norway
     if 56 <= latitude < 64 and 3 <= longitude < 12:
         return 32
-
+    # Special zones for Svalbard
     if 72 <= latitude <= 84 and longitude >= 0:
         if longitude < 9:
             return 31
@@ -323,4 +337,13 @@ def latlon_to_zone_number(latitude, longitude):
 
 
 def zone_number_to_central_longitude(zone_number):
+    check_valid_zone_number(zone_number)
     return (zone_number - 1) * 6 - 180 + 3
+
+def zone_letter_to_central_latitude(zone_letter):
+    check_valid_zone_letter(zone_letter)
+    zone_letter = zone_letter.upper()
+    if zone_letter == 'X':
+        return 78
+    else:
+        return -76 + (ZONE_LETTERS.index(zone_letter) * 8)
